#!/usr/bin/python
# -*- coding:utf-8 -*-
#
#   Copyright 2008 Le Coz Florent <louizatakk@fedoraproject.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the WTF Public License as published by
#   Sam Hocevar.
#
#   DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#   Version 2, December 2004
#   DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
#   0. You just DO WHAT THE FUCK YOU WANT TO.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import gtk
from display_card import *
from ludimagia import *
import re
import sys
import os

def get_info_from_lines(lines):
    '''from the lines representing A single card, return a Card object'''
    ## TODO: could be shorter and faster (and needs to be, for it is called very often)
    ##
    ## pop() is used to get the next line in a simple way (with no line counter etc)
    ## that's why the interresting line is always lines[0]

    info = {}
    info['name'] = lines.pop(0)
    ## we could use a specific color for the lands, instead. (like brown or anything)
    ## but it's cool anyway
    if 'Land' in lines[0]:
        info['cost'] = ''
        if info['name'].strip() == 'Mountain' or info['name'] == 'Montagne':
            info['color'] = 'R'
        elif info['name'] == 'Swamp' or info['name'] == 'Marais':
            info['color'] = 'B'
        elif info['name'] == 'Plains' or info['name'] == 'Plaine':
            info['color'] = 'W'
        elif info['name'] == 'Forest' or info['name'] == 'Forêt':
            info['color'] = 'G'
        elif info['name'] == 'Island' or info['name'] == 'Île':
            info['color'] = 'U'
        else:
            info['color'] = 'Lnd'
        info['type'] = lines[0]
        lines.pop(0)
    else:
        info['cost'] = lines[0]
        color = ''
        for letter in info['cost']:
            if letter in 'RUWGB' and letter not in color:
                color += letter
        if len(color) > 1:
            color = 'Gld'
        if len(color) == 0:
            color = 'Art'
        info['color'] = color
        lines.pop(0)
        info['type'] = lines[0]
        lines.pop(0)
    if len(lines) == 0:
        return info
    pattern = re.compile("^([0-9][0-9]?((�|b)|(\+\*))?|\*)/([0-9][0-9]?((a|b)|(\+\*))?|\*)$")
    if pattern.search(lines[0]):
        info['pwr'] = lines[0]
        lines.pop(0)
    else:
        info['pwr'] = ''
    if len(lines) == 0:
        return info
    if lines[0] in ('Rare', 'Unco', 'Co'):
        info['Rarity'] = lines[0]
        lines.pop(0)
    else:
        info['Rarity'] = 'None'
    text = ''
    while len(lines) >= 1:
        text += lines[0]+'@'
        lines.pop(0)
    info['text'] = text
    return info

def get_list_all_cards():
    ''' from each file in deck/ dir, read all lines and return
    a list of Card objects'''
    list_all_cards = []
    for root, dirs, files in os.walk(CARDS_DIR):
        if '.svn' in dirs:
            dirs.remove('.svn')
        for file in files:
            fd = open(root+file, 'r')
            while True:
                line = fd.readline().strip()
                lines = []
                while line != '':
                    lines.append(line)
                    line = fd.readline().strip()
                if len(lines) != 0:
                    info = get_info_from_lines(lines)
                    card = Card(info)
                    list_all_cards.append(card)
                else:
                    break
        return list_all_cards

class CardList(gtk.TreeView):
    '''a Treeview displaying a list of cards'''
    def __init__(self, file=None, reorderable=True, group=False):
        if group == False:
            self.liststore = gtk.ListStore(str, str, str, str, str, str, str, str)
        else:
            self.liststore = gtk.ListStore(str, str, str, str, str, str, str, str, str)
        list_all_cards = []
        info = {'1': '1'}
        self.list = ['']
        if file: # if a file is True, read the files
            self.list = get_list_all_cards()
            for row in self.list:
                self.liststore.append(row.info)
        else: # if file is False, we have an empty list
            self.list = ['']

        gtk.TreeView.__init__(self, self.liststore)

        self.set_fixed_height_mode(0)
        self.column = gtk.TreeViewColumn('Name')
#        self.column2 = gtk.TreeViewColumn('Color')
        self.column3 = gtk.TreeViewColumn('Cost')
        self.column4 = gtk.TreeViewColumn('Type')
        self.column5 = gtk.TreeViewColumn('Power')
        if group == True:
            self.column6 = gtk.TreeViewColumn('Nb')

        if group == True:
            self.append_column(self.column6)
        self.append_column(self.column)
 #       self.append_column(self.column2)
        self.append_column(self.column3)
        self.append_column(self.column4)
        self.append_column(self.column5)

        self.cell = gtk.CellRendererText()
        self.cell.set_property('width', 170)
        self.cell2 = gtk.CellRendererText()
        self.cell2.set_property('width', 12)
        self.cell3 = gtk.CellRendererText()
        self.cell3.set_property('width', 68)
        self.cell4 = gtk.CellRendererText()
        self.cell4.set_property('width', 250)
        self.cell5 = gtk.CellRendererText()
#        self.cell5.set_property('width', 2)
        if group == True:
            self.cell6 = gtk.CellRendererText()

        self.column.pack_start(self.cell, True)
 #       self.column2.pack_start(self.cell2, True)
        self.column3.pack_start(self.cell3, True)
        self.column4.pack_start(self.cell4, True)
        self.column5.pack_start(self.cell5, True)
        if group == True:
            self.column6.pack_start(self.cell6, True)

        self.column.set_attributes(self.cell, text=0)
  #      self.column2.set_attributes(self.cell2, text=2)
        self.column3.set_attributes(self.cell3, text=3)
        self.column4.set_attributes(self.cell4, text=4)
        self.column5.set_attributes(self.cell5, text=6)
        if group == True:
            self.column6.set_attributes(self.cell6, text=8)
        self.set_search_column(0)
        self.set_reorderable(reorderable)
        self.column.set_sort_column_id(-1)
        self.show()

class Icon_Bar(gtk.VBox):
    '''The vertical bar where icons are displayed'''
    def __init__(self):
        self.selected = None
        gtk.VBox.__init__(self)
        self.but_add = gtk.Button(stock=gtk.STOCK_ADD)
        self.but_del = gtk.Button(stock=gtk.STOCK_REMOVE)
        self.but_clear = gtk.Button(stock= gtk.STOCK_CLEAR)
        self.but_open = gtk.Button(stock= gtk.STOCK_OPEN)
        self.but_save = gtk.Button(stock= gtk.STOCK_SAVE)

        self.pack_end(self.but_add, False, False, 6)
        self.pack_end(self.but_del, False, False, 6)
        self.pack_end(self.but_clear, False, False, 6)
        self.pack_end(self.but_open, False, False, 6)
        self.pack_end(self.but_save, False, False, 6)

class Deck_Stats(gtk.VBox):
    '''The box where information about the current deck are
    displayed'''
    def __init__(self):
        gtk.VBox.__init__(self)

        red_vbox = gtk.HBox()
        r_icon = gtk.Image()
        r_icon.set_from_file('images/R.png')
        self.red_per = gtk.Label('0%')
        red_vbox.pack_start(r_icon)
        red_vbox.pack_start(gtk.Label('Red: '))
        red_vbox.pack_start(self.red_per)
        self.pack_start(red_vbox)

        blue_vbox = gtk.HBox()
        u_icon = gtk.Image()
        u_icon.set_from_file('images/U.png')
        self.blue_per = gtk.Label('0%')
        blue_vbox.pack_start(u_icon)
        blue_vbox.pack_start(gtk.Label('Blue: '))
        blue_vbox.pack_start(self.blue_per)
        self.pack_start(blue_vbox)

        green_vbox = gtk.HBox()
        g_icon = gtk.Image()
        g_icon.set_from_file('images/G.png')
        self.green_per = gtk.Label('0%')
        green_vbox.pack_start(g_icon)
        green_vbox.pack_start(gtk.Label('Green: '))
        green_vbox.pack_start(self.green_per)
        self.pack_start(green_vbox)

        black_vbox = gtk.HBox()
        b_icon = gtk.Image()
        b_icon.set_from_file('images/B.png')
        self.black_per = gtk.Label('0%')
        black_vbox.pack_start(b_icon)
        black_vbox.pack_start(gtk.Label('Black: '))
        black_vbox.pack_start(self.black_per)
        self.pack_start(black_vbox)

        white_vbox = gtk.HBox()
        w_icon = gtk.Image()
        w_icon.set_from_file('images/W.png')
        self.white_per = gtk.Label('0%')
        white_vbox.pack_start(w_icon)
        white_vbox.pack_start(gtk.Label('White: '))
        white_vbox.pack_start(self.white_per)
        self.pack_start(white_vbox)

        art_vbox = gtk.HBox()
        a_icon = gtk.Image()
        a_icon.set_from_file('images/X.png')
        self.art_per = gtk.Label('0%')
        art_vbox.pack_start(a_icon)
        art_vbox.pack_start(gtk.Label('Artifact: '))
        art_vbox.pack_start(self.art_per)
        self.pack_start(art_vbox)

        tot_vbox = gtk.HBox()
        self.total = gtk.Label('0')
        tot_vbox.pack_start(gtk.Label('Total: '))
        tot_vbox.pack_start(self.total)
        tot_vbox.pack_start(gtk.Label(' cards'))
        self.pack_start(tot_vbox)

class Card(object):
    '''A card object
    dicinfo is a dictionnary, info is a list
    both contains the same information
    (I really wonder why we need 'info',
    as dicinfo is much more simple)'''
    def __init__(self, info):
       self.dicinfo = info
       self.info = []
       try: self.info.append(info['name'])
       except:self.info.append('Name')
       try: self.info.append(info['edition'])
       except:self.info.append('')
       try: self.info.append(info['color'])
       except:self.info.append('Color')
       try: self.info.append(info['cost'])
       except:self.info.append('')
       try: self.info.append(info['type'])
       except:self.info.append('Type')
       try: self.info.append(info['text'])
       except:self.info.append('')
       try: self.info.append(info['pwr'])
       except:self.info.append('None')
       try: self.info.append(info['rarity'])
       except:self.info.append('None')
       return

class DeckCreator(gtk.Window):
    '''The big interface where we build, edit, and view decks'''
    def __init__(self, file):
        gtk.Window.__init__(self)
        self.set_title('Ludimagia - Deck Creator v%s' % (VERSION))
        self.set_icon_from_file('images/logo.png')
        self.connect("destroy", gtk.main_quit)

        self.set_border_width(0)
        self.set_default_size(1600, 800)

        self.tv_deck = CardList(None, True, True)
        self.tv_deck.connect('cursor-changed', self.on_cursor_changed, self.tv_deck.get_selection())
        self.tv_cardlist = CardList(file, False, False)
        self.tv_cardlist.connect('cursor-changed', self.on_cursor_changed, self.tv_cardlist.get_selection())
        scrolled1 = gtk.ScrolledWindow()
        scrolled1.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled1.add(self.tv_cardlist)
        scrolled2 = gtk.ScrolledWindow()
        scrolled2.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled2.add(self.tv_deck)
        self.icon_bar = Icon_Bar()
        self.icon_bar.but_add.connect('clicked', self.on_add_to_deck)
        self.icon_bar.but_del.connect('clicked', self.on_rem_from_deck)
        self.icon_bar.but_clear.connect('clicked', self.on_clear_deck)
        self.icon_bar.but_open.connect('clicked', self.on_open_deck)
        self.icon_bar.but_save.connect('clicked', self.on_save_deck)

        self.stats = Deck_Stats()
        self.stats.show_all()
        self.pic = CardPic()

        hb = gtk.HBox()
        hb.pack_start(self.pic)
        hb.pack_start(self.stats)

        vbox = gtk.VBox()
        vbox.pack_start(hb, False, False, 0)
        vbox.pack_start(scrolled2)

        hbox = gtk.HBox()
        hbox.pack_start(scrolled1)
        hbox.pack_start(self.icon_bar, False, False, 0)
        hbox.pack_start(vbox)

        self.add(hbox)

        self.show_all()

    def on_save_deck(self, but=None):
        '''Saves deck in a file
        in a Apprentice format by default'''
        store = self.tv_deck.liststore
        chooser = gtk.FileChooserDialog(title=None,action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                 buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_SAVE,gtk.RESPONSE_OK))
        chooser.show()
        rep = chooser.run()
        if rep == gtk.RESPONSE_OK:
            file = chooser.get_filename()
            fd = open(file, 'w')
            fd.write('// Deck file for ludimagia (http://codingteam.net/project/ludimagia)\n\n')
            store.foreach(self.write_in_file, fd)
            fd.close()
            chooser.destroy()
        else:
            chooser.destroy()
            return False
        return True

    def write_in_file(self, model, path, iter, fd):
        '''Function called for each line (with .foreach) in the deck treeview
        write the line in the file'''
        name = self.tv_deck.liststore.get_value(iter, 0)
        nb = self.tv_deck.liststore.get_value(iter, 8)
        fd.write('    '+str(nb)+' '+name+'\n')

    def on_open_deck(self, but=None):
        '''Open an existing deck
        clean the deck treeview and put the opened deck instead'''
        chooser = gtk.FileChooserDialog(title='Open an existing Deck', action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                  buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*");

        chooser.add_filter(filter)
        chooser.show()
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            file = chooser.get_filename()
            fd = open(file, 'r')
            lines = fd.readlines()
            for line in lines: ## check file format
                ## open both Apprentice and MWS format (atm)
                if line.startswith('//') or line.startswith('SB:') or line.startswith(' -') or len(line.strip()) == 0:
                    ## ignore the Sideboard (DeckCreator doesn't handle that) and the comments
                    continue
                elif line.startswith('    ') == False:
                    err_dial = gtk.Dialog('File format error', None, buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
                    err_dial.show()
                    rep = err_dial.run()
                    err_dial.destroy()
                    chooser.destroy()
                    fd.close()
                    return False
            ## file format is ok, then clear the deck to display the new one
            self.tv_deck.liststore.clear()
            tmp_list = [] ## list used to detect duplicate cards. They should be ignored.
            for line in lines:
                if line.startswith('//') or line.startswith('SB:') or line.startswith(' -') or len(line.strip()) == 0:
                    continue
                else:
                    li = line.strip().split(' ')
                    if li[1].startswith('['):
                        name = ' '.join(li[2:])
                    else:
                        name = ' '.join(li[1:])
                    if name.find('(') != -1:
                        name = name[:name.find('(')-1]
                    nb = li[0]
                    if name not in tmp_list:
                        self.tv_cardlist.liststore.foreach(self.search_and_add_to_deck, (nb, name))
                        tmp_list.append(name)
            chooser.destroy()
            fd.close()
            self.update() ## update the statistics of the deck
            return True
        else:
            chooser.destroy()
            return False

    def search_and_add_to_deck(self, model, path, iter, user_data):
        '''Function called on each line (with .foreach) of the cards_list (which may
        be very huge, that's why opening a deck takes so many time) to find
        the card's informations and put everything in the deck treeview'''
        (nb, name) = user_data
        if self.tv_cardlist.liststore.get_value(iter, 0) == name:
            liststore = self.tv_cardlist.liststore
            edition = liststore.get_value(iter, 1)
            color = liststore.get_value(iter, 2)
            cost = liststore.get_value(iter, 3)
            type = liststore.get_value(iter, 4)
            desc = liststore.get_value(iter, 5)
            power = liststore.get_value(iter, 6)
            rarity = liststore.get_value(iter, 7)
            self.tv_deck.liststore.append((name, edition, color, cost, type, desc, power, rarity, nb))
            return True

    def on_clear_deck(self, but=None):
        '''clear the deck treeview (is this really useful ?...)'''
        selection = self.tv_deck.get_selection()
        liststore, iter = selection.get_selected()

        liststore.clear()
        self.update()

    def on_cursor_changed(self, other, selection):
        '''function called on each cursor change'''
        liststore, iter = selection.get_selected()
        try:
            self.pic.color = liststore.get_value(iter, 2)
            info = liststore.get(iter, 0, 1, 2, 3, 4, 5, 6)
            self.pic.info = info
        except:pass

        self.pic.area_expose(self.pic)

    def on_rem_from_deck(self, but=None):
        '''remove the selected card from the deck treeview'''
        selection = self.tv_deck.get_selection()
        liststore, iter = selection.get_selected()

        if type(iter) != gtk.TreeIter: # if no card is selected in the deck
            return
        nb_card = int(self.tv_deck.liststore.get_value(iter, 8))
        if nb_card > 1:
            self.tv_deck.liststore.set_value(iter, 8, nb_card - 1)
        else:
            path = liststore.get_path(iter)
            liststore.remove(iter)

            if path[0] == 0:
                prev = path[0]
            else:
                prev = path[0] - 1
            selection.select_path(prev)
        self.update()


    def on_add_to_deck(self, but=None):
        'add selected card from list treeview to deck treeview'''
        selection = self.tv_cardlist.get_selection()
        liststore, iter = selection.get_selected()

        name = liststore.get_value(iter, 0)
        edition = liststore.get_value(iter, 1)
        color = liststore.get_value(iter, 2)
        cost = liststore.get_value(iter, 3)
        type = liststore.get_value(iter, 4)
        desc = liststore.get_value(iter, 5)
        power = liststore.get_value(iter, 6)
        rarity = liststore.get_value(iter, 7)
        nb = '1'
        self.nb_card = 0
        self.tv_deck.liststore.foreach(self.increment_number, (name, edition, color, cost, type, desc, power, rarity, 0))
        if self.nb_card == 0:
            self.tv_deck.liststore.append((name, edition, color, cost, type, desc, power, rarity, nb))
        self.update()

    def update(self):
        '''update the deck's stats, according to what is in the deck treeview'''
        self.st = {'R':0, 'U':0, 'B':0, 'W':0, 'G':0, 'Gld':0, 'Art':0, 'Lnd':0, 'tot':0}
        self.tv_deck.liststore.foreach(self.update_stats, None)
        if self.st['tot'] == 0:
            return
        self.stats.total.set_text(str(self.st['tot']))
        self.stats.red_per.set_text(str(round(self.st['R']/self.st['tot']*100))+'%')
        self.stats.blue_per.set_text(str(round(self.st['U']/self.st['tot']*100))+'%')
        self.stats.green_per.set_text(str(round(self.st['G']/self.st['tot']*100))+'%')
        self.stats.black_per.set_text(str(round(self.st['B']/self.st['tot']*100))+'%')
        self.stats.white_per.set_text(str(round(self.st['W']/self.st['tot']*100))+'%')
        self.stats.art_per.set_text(str(round(self.st['Art']/self.st['tot']*100))+'%')

    def update_stats(self, model, path, iter, user_data):
        color = self.tv_deck.liststore.get_value(iter, 2)
        nb = self.tv_deck.liststore.get_value(iter, 8)
        self.st[color] += float(nb)
        self.st['tot'] += float(nb)

    def increment_number(self, model, path, iter, user_data):
        '''increment number of cards in the treeview, instead of adding
        a line, if the card is already in the deck treeview'''
        name = self.tv_deck.liststore.get_value(iter, 0)
        nb = self.tv_deck.liststore.get_value(iter, 8)
        if name == user_data[0]:
            self.tv_deck.liststore.set_value(iter, 8, str(int(nb) + 1))
            self.nb_card += 1
            return True

    def set_deck_size(self, model, path, iter, user_data):
        self.deck_size += 1

if __name__ == '__main__':
    dc = DeckCreator('card_list.txt')
    gtk.main()
    exit()
